package fyloz.trial.ColorRecipesExplorer.web.controller.creators;

import fyloz.trial.ColorRecipesExplorer.model.Material;
import fyloz.trial.ColorRecipesExplorer.services.MaterialService;
import fyloz.trial.ColorRecipesExplorer.services.MaterialTypeService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.http.MediaType;
import org.springframework.stereotype.Controller;
import org.springframework.ui.Model;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.multipart.MultipartFile;

import javax.validation.Valid;

import static fyloz.trial.ColorRecipesExplorer.web.PagesPaths.CREATOR_MATERIAL;
import static fyloz.trial.ColorRecipesExplorer.web.PagesPaths.CREATOR_MATERIAL_SUCCESS;
import static fyloz.trial.ColorRecipesExplorer.web.StringBank.*;

@Controller
public class MaterialCreatorController {

    private MaterialService materialService;
    private MaterialTypeService materialTypeService;

    @Autowired
    public MaterialCreatorController(MaterialService materialService, MaterialTypeService materialTypeService) {
        this.materialService = materialService;
        this.materialTypeService = materialTypeService;
    }

    /**
     * Affiche la page de création d'un produit.
     *
     * @return La page à afficher.
     */
    @GetMapping(CREATOR_MATERIAL)
    public String showCreationPage(Model model, Material material) {
        model.addAttribute(MATERIAL, material == null ? new Material() : material);
        model.addAttribute(MATERIAL_TYPES, materialTypeService.getAll());
        return CREATOR_MATERIAL;
    }

    /**
     * Permet à l'utilisateur de créer un produit.
     * <p>
     * La création échouera si:
     * - L'utilisateur n'est pas autorisé à effectuer cette action
     * - Le produit existe déjà
     * - Une erreur est survenue lors de la sauvegarde dans la base de données
     * <p>
     * Modèle de la page:
     * - error: Contient le message d'erreur, s'il y a lieu
     * - materialCode: Contient le code de produit du produit créé
     * <p>
     * REQUIERT UNE AUTORISATION
     *
     * @param model    Le modèle injecté par Thymeleaf
     * @param material Le produit à créer
     * @param simdut   Le fichier SIMDUT du produit (optionnel)
     * @return La page à afficher.
     */
    @PostMapping(value = CREATOR_MATERIAL, consumes = MediaType.MULTIPART_FORM_DATA_VALUE)
    public String create(Model model, @Valid Material material, MultipartFile simdut) {
        if (!materialService.exists(material)) {
            if ((material = materialService.save(material)) != null) {
                model.addAttribute(MATERIAL_CODE, material.getMaterialCode());

                if (!materialService.addSimdut(simdut, material)) {
                    model.addAttribute(RESPONSE_ERROR, ERROR_SAVING_SIMDUT);
                }

                return CREATOR_MATERIAL_SUCCESS;
            } else {
                model.addAttribute(RESPONSE_ERROR, ERROR_SAVING);
            }
        } else {
            model.addAttribute(RESPONSE_ERROR, String.format(MATERIAL_ALREADY_EXIST, material.getMaterialCode()));
        }

        return showCreationPage(model, material);
    }
}
