package fyloz.trial.ColorRecipesExplorer.web.controller.creators;

import fyloz.trial.ColorRecipesExplorer.model.Recipe;
import fyloz.trial.ColorRecipesExplorer.services.CompanyService;
import fyloz.trial.ColorRecipesExplorer.services.RecipeService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Controller;
import org.springframework.ui.Model;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PostMapping;

import javax.validation.Valid;

import static fyloz.trial.ColorRecipesExplorer.web.PagesPaths.CREATOR_RECIPE;
import static fyloz.trial.ColorRecipesExplorer.web.PagesPaths.CREATOR_RECIPE_SUCCESS;
import static fyloz.trial.ColorRecipesExplorer.web.StringBank.*;

@Controller
public class RecipeCreatorController {

    private RecipeService recipeService;
    private CompanyService companyService;

    @Autowired
    public RecipeCreatorController(RecipeService recipeService, CompanyService companyService) {
        this.recipeService = recipeService;
        this.companyService = companyService;
    }

    /**
     * Affiche la page de création d'une recette.
     *
     * @param model Le Model injecté par Thymeleaf
     * @return La page à afficher.
     */
    @GetMapping(CREATOR_RECIPE)
    public String showCreationPage(Model model) {
        model.addAttribute(COMPANIES, companyService.getAll());

        return CREATOR_RECIPE;
    }

    /**
     * Permet à l'utilisateur de créer une nouvelle recette.
     * <p>
     * La création échouera si:
     * - L'utilisateur n'est pas autorisé à exécuter cette action
     * - Une erreur est survenue lors de la sauvegarde dans la base de données
     * <p>
     * Modèle de la page:
     * - error: Contient le message d'erreur, s'il y a lieu
     * <p>
     * REQUIERT UNE AUTORISATION
     *
     * @param model  Le Model injecté par Thymeleaf
     * @param recipe La recette à créer
     * @return La page à afficher.
     */
    @PostMapping(value = CREATOR_RECIPE)
    public String createRecipe(Model model, @Valid Recipe recipe) {
        if ((recipe = recipeService.save(recipe)) != null) {
            model.addAttribute(RECIPE_CODE, recipe.getRecipeCode());
            model.addAttribute(RECIPE_ID, recipe.getRecipeID());
            return CREATOR_RECIPE_SUCCESS;
        } else {
            model.addAttribute(RESPONSE_ERROR, ERROR_SAVING);
            return showCreationPage(model);
        }
    }
}
