package fyloz.trial.ColorRecipesExplorer.web.controller.removers;

import fyloz.trial.ColorRecipesExplorer.model.Material;
import fyloz.trial.ColorRecipesExplorer.services.MaterialService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Controller;
import org.springframework.ui.Model;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.PostMapping;

import java.util.stream.Collectors;

import static fyloz.trial.ColorRecipesExplorer.web.PagesPaths.REMOVER_MATERIAL;
import static fyloz.trial.ColorRecipesExplorer.web.PagesPaths.REMOVER_MATERIAL_SPECIFIC;
import static fyloz.trial.ColorRecipesExplorer.web.StringBank.*;

@Controller
public class MaterialRemoverController {

    private MaterialService materialService;

    @Autowired
    public MaterialRemoverController(MaterialService materialService) {
        this.materialService = materialService;
    }

    /**
     * Affiche la page de suppression des produits
     *
     * @param model Le Model injecté par Thymeleaf
     * @return La page à afficher.
     */
    @GetMapping(REMOVER_MATERIAL)
    public String showPage(Model model) {
        model.addAttribute(MATERIALS, materialService.getAll().stream().filter(m -> !m.isMixType()).collect(Collectors.toList()));

        return REMOVER_MATERIAL;
    }

    /**
     * Permet à l'utilisateur de supprimer un produit.
     * Cette méthode requiert l'identifiant du produit dans l'URL.
     * <p>
     * La suppression échouera si:
     * - L'utilisateur n'est pas autorisé à exécuter cette action
     * - Le produit n'existe pas
     * - Le produit est lié à d'autres mélanges
     * <p>
     * Modèle de la page:
     * - error: Contient le message d'erreur, s'il y a lieu
     * - materialCode: Contient le code de produit du produit supprimé
     * <p>
     * REQUIERT UNE AUTORISATION
     *
     * @param model      Le Model injecté par Thymeleaf
     * @param materialID L'identifiant du produit à supprimer
     * @return La page à afficher.
     */
    @PostMapping(REMOVER_MATERIAL_SPECIFIC)
    public String removeMaterial(Model model, @PathVariable int materialID) {
        Material material = materialService.getByID(materialID);

        if (material == null) {
            model.addAttribute(RESPONSE_ERROR, String.format(MATERIAL_NOT_FOUND, materialID));
        } else {
            if (materialService.deleteIfNotLinked(material)) {
                model.addAttribute(MATERIAL_CODE, material.getMaterialCode());

                if (!materialService.removeSimdut(material)) {
                    model.addAttribute(RESPONSE_ERROR, ERROR_SAVING_SIMDUT);
                }
            } else {
                model.addAttribute(RESPONSE_ERROR, String.format(MATERIAL_LINKED, material.getMaterialCode()));
            }
        }

        return showPage(model);
    }
}
